class SignalData(object):
    """ Object intended to store information about received signals """

    def __init__(self, signum, received=False, frame=None, acknowledged=False):
        self.signum = signum
        self.received = received
        self.frame = frame
        self.acknowledged = acknowledged

    def awk(self):
        """ Acknowledge the signal, this suppresses it in the feed """
        self.acknowledged = True
        return self

    def rec(self, frame):
        """ Receive a the signal, this raises it in the feed """
        self.received = True
        self.acknowledged = False
        self.frame = frame
        return self

    def __bool__(self):
        return self.received and not self.acknowledged

    def __nonzero__(self):
        return self.__bool__()



import signal

from .signalData import SignalData


class Feed(object):
    """ Object intended to raise signals for visibility """

    def __init__(self):
        self.signals = dict()

    def _receiver(self, signum, frame):
        """ Callback that is registered in the signal.signal builtin """
        self.signals[signum].rec(frame)

    def __getitem__(self, item):
        if item not in self.signals:
            signal.signal(item, self._receiver)
            self.signals[item] = SignalData(item)
        return self.signals.__getitem__(item)

    def __bool__(self):
        return any([sig for sig in self.signals.values()])

    def __nonzero__(self):
        return self.__bool__()



from .feed import Feed

feed = None


def received(signal_id, awk=True):
    """
    This method checks if the provided signal has been received,
    and returns a Boolean.

    The first call per signal_id will start up a listener for that signal.
    All calls after that check the state of that listener.

    :param Integer signal_id: The signal to check.
        It is recommended to pull these from the signals builtin.
    :param Boolean awk: Optional, defaults to True.
        True will acknowledge the signal, so that the same signal does not
        cause this return True twice for the same signal.
        False skips this step.
    :return Boolean: True if the signal has been received.  Else False.
    """
    global feed
    if feed is None:
        feed = Feed()
    if not feed[signal_id]:
        return False
    if awk:
        feed[signal_id].awk()
    return True



# -*- coding: utf-8 -*-

from __future__ import print_function

import logging
import signal
import sys
import time

from .wrappers import received


logging.basicConfig(
    level=logging.INFO, format='[ PID %(process)d ] %(message)s')

symbols = [
    u'    →     ',
    u'     →    ',
    u'      →   ',
    u'       →  ',
    u'        → ',
    u'         →',
    u'         ↘',
    u'         ↓',
    u'         ↙',
    u'         ←',
    u'        ← ',
    u'       ←  ',
    u'      ←   ',
    u'     ←    ',
    u'    ←     ',
    u'   ←      ',
    u'  ←       ',
    u' ←        ',
    u'←         ',
    u'↖         ',
    u'↑         ',
    u'↗         ',
    u'→         ',
    u' →        ',
    u'  →       ',
    u'   →      ',
    u'    →     ',
    u'     ↷    ',
    u'     ↻    ',
    u'     ⟳    ',
]


def trick():
    """ Spin the spinner """
    for symbol in symbols:
        print(symbol, end='\r')
        sys.stdout.flush()
        time.sleep(.1)


def main():
    """ A demo that can be used to test functionality """
    logging.info('Hello, send me a SIGTERM to exit, or a SIGHUP for a trick')
    while not received(signal.SIGTERM):
        if received(signal.SIGHUP):
            trick()
            logging.info('Neat huh?')
        else:
            time.sleep(.5)
    logging.info('See you later')



import versioneer
from setuptools import setup


with open('README.rst', 'r') as fh:
    long_description = fh.read()


setup(
    name='aerial',
    packages=['aerial'],
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description='The easy way to catch signals',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author='Chris Brake',
    author_email='chris.brake@gmail.com',
    url='https://github.com/chrisbrake/aerial',
    keywords=['aerial', 'signals', 'signal', 'unix', 'sighup', 'sigterm'],
    classifiers=[
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries'
    ],
)
